"""Compare outputs to those which are computed by the python bindings of FN5
"""
import pytest
import fn5

def test_1():
    '''This should be all comparisons
    '''
    with open("test/all.txt") as f:
        paths = [line.strip() for line in f]
    ref = fn5.load_reference("NC_045512.fasta")
    samples = [fn5.Sample(path, ref, set(), "sample"+path.split(".")[0][-1]) for path in paths]

    actual = set(fn5.compute(samples))
    actual_map = {}
    for a, b, dist in actual:
        actual_map[(a, b)] = dist
        actual_map[(b, a)] = dist
    expected = [
        "sample4 sample1 12",
        "sample4 sample3 11",
        "sample4 sample2 11",
        "sample1 sample3 1",
        "sample1 sample2 1",
        "sample3 sample2 0",
        ]
    expected = set([(x.split(" ")[0], x.split(" ")[1], int(x.split(" ")[2])) for x in expected])
    expected_map = {}
    for a, b, dist in expected:
        expected_map[(a, b)] = dist
        expected_map[(b, a)] = dist
    
    assert actual_map == expected_map


import pytest

'''All files are parsed into a set to avoid order based issues due to multithreading
'''

def test_1():
    '''This should be all comparisons
    '''
    with open("test/output/1.txt") as f:
        actual = set([tuple(sorted(line.strip().split(" "))) for line in f])
    
    expected = [
        "sample4 sample1 12",
        "sample4 sample3 11",
        "sample4 sample2 11",
        "sample1 sample3 1",
        "sample1 sample2 1",
        "sample3 sample2 0",
        ]
    expected = set([tuple(sorted(x.split(" "))) for x in expected])
    
    assert actual == expected

def test_2():
    '''This should be adding a single sample
    '''
    with open("test/output/2.txt") as f:
        actual = set([tuple(sorted(line.strip().split(" "))) for line in f])
    
    expected = set([
        "sample4 sample1 12",
        "sample4 sample3 11",
        "sample4 sample2 11",
        ])
    expected = set([tuple(sorted(x.split(" "))) for x in expected])

    assert actual == expected

def test_3():
    '''This should be adding samples 1 and 2
    '''
    with open("test/output/3.txt") as f:
        actual = set([tuple(sorted(line.strip().split(" "))) for line in f])
    
    expected = set([
        "sample1 sample4 12",
        "sample3 sample1 1",
        "sample3 sample4 11",
        "sample2 sample1 1",
        "sample2 sample4 11",
        ])
    expected = set([tuple(sorted(x.split(" "))) for x in expected])

    assert actual == expected

def test_4():
    '''This should just be adding sample4
    '''
    with open("test/output/4.txt") as f:
        actual = set([tuple(sorted(line.strip().split(" "))) for line in f])

    expected = [
            "sample4 sample1 12",
            "sample4 sample3 11",
            "sample4 sample2 11",
        ]
    expected = set([tuple(sorted(x.split(" "))) for x in expected])
    assert actual == expected


# Based on example https://github.com/pybind/python_example
from pybind11.setup_helpers import Pybind11Extension
from setuptools import setup
from pathlib import Path

__version__ = "1.2.0rc7"

try:
    this_directory = Path(__file__).parent
    long_description = (this_directory / "PYTHON_README.md").read_text()
except FileNotFoundError:
    # Issues around packaging other readme means we can just skip on install
    long_description = "Fast, scalable SNP distance calculation from disk."

ext_modules = [
    Pybind11Extension(
        "fn5",
        sorted(["src/sample.cpp", "src/comparisons.cpp", "src/fn5_python.cpp"]),
        include_dirs=["src/include"],
        extra_compile_args=["-std=c++20", "-O3", "-pthread"],
        # Example: passing in the version to the compiled code
        define_macros=[("VERSION_INFO", __version__)],
    ),
]

setup(
    name="fn5",
    version=__version__,
    author="Jeremy Westhead",
    author_email="jeremy.westhead@ndm.ox.ac.uk",
    url="https://github.com/oxfordmmm/FN5",
    description="Fast, scalable SNP distance calculation from disk.",
    ext_modules=ext_modules,
    zip_safe=False,
    python_requires=">=3.7",
    license="University of Oxford, see LICENSE",
    long_description=long_description,
    long_description_content_type='text/markdown',
    include_package_data=True,
    package_data={'yourpackage': ['*']}
)